/*
 * AKE2008 by Murachue
 * 640x480
 * init: 20071230
 */

#include <windows.h>
#include <stdio.h>
#include <mbstring.h>
#include <math.h>
#include "wavemix/wavemix.h"
#include "inc/fmod.h"
#include "resource.h"

#define PI 3.141592653589

struct FontCharPoly{
	BYTE points;
	POINT* point;
};
struct FontChar{
	BYTE polys;
	struct FontCharPoly* poly;
};

struct MainWndVal{
	HWND hWnd;

	// GraphicSystem
	HDC hBackBuffer;
	HBITMAP hFirstBackBufferBitmap;
	SIZE wsize;

	// InternalSystem
	UINT phase;
	UINT counter;
	BOOL outmouse;
	POINT mouse;
	struct FontChar font[256];

	// GameSystem
	struct{
		double x;
		double y;
		double at;	// AccelTheta
		double af;	// AccelForce
	}me;
	struct WireRock{
		BOOL use;

		double x;
		double y;
		double r;
		double bump[16];	// dekoboko
		double rofs;
		double arofs;
		double ax;
		double ay;
	}rock[128];
	struct BombWind{
		BOOL use;

		double x;
		double y;
		double r;
		double maxr;

		BOOL plyrwind;
		BOOL inval;
	}wind[128];

	UINT CurrentFont;

	struct{
		PWAVEDATA FireMain;
		PWAVEDATA Accel;
		PWAVEDATA Bomb;
		PWAVEDATA RenBomb;
		PWAVEDATA BoomWarn;
		PWAVEDATA Invalid;
		PWAVEDATA Guard;
		PWAVEDATA NExp;
		PWAVEDATA FExp;
		PWAVEDATA Chain;
		PWAVEDATA Life[5];
		PWAVEDATA ChainBonus;
		PWAVEDATA GameOver;
		PWAVEDATA GuardedBase;
		PWAVEDATA QuickSweep;
		PWAVEDATA LongSweep;
	}wav;
	struct{
		FMUSIC_MODULE* Main;
	}bgm;

	// Param
	UINT vk;
};

void (*PhaseProcess[])(UINT msg, struct MainWndVal* mwv);
void SetPhase(struct MainWndVal* mwv, UINT phase)
{
	PhaseProcess[mwv->phase](WM_DESTROY, mwv);
	mwv->phase = phase;
	mwv->counter = 0;
	PhaseProcess[mwv->phase](WM_CREATE, mwv);

	return;
}
BOOL ClipCursorToWindowClient(HWND hWnd, BOOL clipit)
{
	RECT rc;

	if(clipit)
	{
		GetClientRect(hWnd, &rc);
		ClientToScreen(hWnd, (POINT*)&rc);
		ClientToScreen(hWnd, (POINT*)&rc + 1);
		return ClipCursor(&rc);
	}else
		return ClipCursor(NULL);
}

void Line(HDC hdc, int x, int y, int x2, int y2)
{
	POINT p;

	MoveToEx(hdc, x, y, &p);
	LineTo(hdc, x2, y2);
	MoveToEx(hdc, p.x, p.y, NULL);

	return;
}
BOOL TextOutAuto(HDC hdc, int x, int y, char* str)
{
	return TextOut(hdc, x, y, str, strlen(str));
}
BOOL TextOutFormat(HDC hdc, int x, int y, char* str, ...)
{
	char ts[1024];
	va_list va;

	va_start(va, str);
	vsprintf(ts, str, va);
	va_end(va);
	return TextOut(hdc, x, y, ts, strlen(ts));
}
void FirePolyline(HDC hdc, POINT* p, int n)	// Polyline includes last point.
{
	MoveToEx(hdc, p->x, p->y, NULL);
	PolylineTo(hdc, p, n);
	LineTo(hdc, p[n - 1].x + 1, p[n - 1].y);
}
void PointSet(HDC hdc, int x, int y)
{
	MoveToEx(hdc, x, y, NULL);
	LineTo(hdc, x + 1, y + 1);
}
void FirePolylineTransXY(HDC hdc, POINT* poly, int n, double zoomx, double zoomy, int x, int y)
{
	POINT* p;
	int j;

	p = malloc(sizeof(POINT) * n);
	for(j = 0; j < n; j++)
	{
		p[j].x = (int)(poly[j].x * zoomx) + x;
		p[j].y = (int)(poly[j].y * zoomy) + y;
	}
	FirePolyline(hdc, p, j);
	free(p);
}
#define FirePolylineTrans(hdc, poly, n, zoom, x, y)	FirePolylineTransXY(hdc, poly, n, zoom, zoom, x, y)
void FontOutTransXY(HDC hdc, int x, int y, double zoomx, double zoomy, struct FontChar* fc)
{
	int i;

	for(i = 0; i < fc->polys; i++)
		FirePolylineTransXY(hdc, fc->poly[i].point, fc->poly[i].points, zoomx, zoomy, x, y);
}
#define FontOut(hdc, x, y, zoom, fc) FontOutTransXY(hdc, x, y, zoom, zoom, fc)
void FontOutStringTransXY(HDC hdc, int x, int y, double zoomx, double zoomy, struct FontChar* font, char* str)
{
	int i, max;

	max = strlen(str);
	for(i = 0; i < max; i++)
		FontOutTransXY(hdc, x + (((int)(8 * zoomx) + 1) * i), y, zoomx, zoomy, &font[str[i]]);
}
#define FontOutString(hdc, x, y, zoom, font, str) FontOutStringTransXY(hdc, x, y, zoom, zoom, font, str)
HANDLE WINAPI CreateFileByExe(LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile)
{
	char ts[1024];
	GetModuleFileName(NULL, ts, sizeof(ts));
	if(_mbsrchr(ts, '\\'))
		*_mbsrchr(ts, '\\') = '\0';
	strcat(ts, "\\");
	strcat(ts, lpFileName);

	return CreateFile(ts, dwDesiredAccess, dwShareMode, lpSecurityAttributes, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile);
}

void mfcpy(void* dst, void** src, unsigned int size)
{
	memcpy(dst, *src, size);
	*(BYTE**)src += size;
}
void LoadFont(char* fname, struct MainWndVal* mwv)
{
	HANDLE hf;

	if(!fname || !strlen(fname))
	{	// from resource
		char* dt = LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_FSYSTEM), "VFNT")));

		int i, j, k;

		for(i = 0; i < 256; i++)
		{
			mfcpy(&mwv->font[i].polys, &dt, sizeof(mwv->font[i].polys));
			mwv->font[i].poly = malloc(mwv->font[i].polys * sizeof(struct FontCharPoly));
			for(j = 0; j < mwv->font[i].polys; j++)
			{
				mfcpy(&mwv->font[i].poly[j].points, &dt, sizeof(mwv->font[i].poly[j].points));
				mwv->font[i].poly[j].point = malloc(mwv->font[i].poly[j].points * sizeof(POINT));
				for(k = 0; k < mwv->font[i].poly[j].points; k++)
				{
					mfcpy(&mwv->font[i].poly[j].point[k], &dt, sizeof(mwv->font[i].poly[j].point[k]));
				}
			}
		}
	}else
	{	// from file
		if(!(hf = CreateFileByExe(fname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)))
			MessageBox(mwv->hWnd, "Internal Error: can't open system.fnt by Read", NULL, 0);
		else
		{
			int i, j, k;
			DWORD read;

			for(i = 0; i < 256; i++)
			{
				ReadFile(hf, &mwv->font[i].polys, sizeof(mwv->font[i].polys), &read, NULL);
				mwv->font[i].poly = malloc(mwv->font[i].polys * sizeof(struct FontCharPoly));
				for(j = 0; j < mwv->font[i].polys; j++)
				{
					ReadFile(hf, &mwv->font[i].poly[j].points, sizeof(mwv->font[i].poly[j].points), &read, NULL);
					mwv->font[i].poly[j].point = malloc(mwv->font[i].poly[j].points * sizeof(POINT));
					for(k = 0; k < mwv->font[i].poly[j].points; k++)
					{
						ReadFile(hf, &mwv->font[i].poly[j].point[k], sizeof(mwv->font[i].poly[j].point[k]), &read, NULL);
					}
				}
			}
			CloseHandle(hf);
		}
	}
}

enum{
	PHASE_NULL = 0
	,	// it's dummy...
	PHASE_FONTEDIT,
	PHASE_LOGO,
	PHASE_TITLE,
	PHASE_GAME,
};

void PP_Null(UINT msg, struct MainWndVal* mwv)
{
	// Yes, I'm NULL!
}
void PP_FontEdit(UINT msg, struct MainWndVal* mwv)
{
	static struct FontCharPoly* curpoly;
	static POINT* curpoint;
	static POINT curpoint_lastchallangepoint;
	static int curpoint_challanges;

	switch(msg)
	{
	case WM_CREATE:
		mwv->CurrentFont = 0;
		curpoly = NULL;
		curpoint = NULL;
		curpoint_lastchallangepoint.x = -1;
		break;
	case WM_PAINT:
		{
			HDC hdc = mwv->hBackBuffer;
			int i, j;

			SetTextColor(hdc, 0xFFFFFF);
			SetBkMode(hdc, TRANSPARENT);

			for(i = 0; i < 17; i++)
			{
				Line(hdc, 0, i * 16, 16 * 16, i * 16);
				Line(hdc, i * 16, 0, i * 16, (i-8<0?16:16-(i-8)) * 16);
			}

			TextOutAuto(hdc, 16 * 16 + 2, 0, "Welcome to the font editor");
			TextOutFormat(hdc, 16 * 16 + 2, 18, "Font No: %d (%c)", mwv->CurrentFont, mwv->CurrentFont);

			for(j = 0; j < 16; j++)
				for(i = 0; i < 16; i++)
					if(mwv->font[j*16+i].polys)
						FontOut(hdc, 16 * 16 + 50 + i*16, 100 + j*16, 1.0, &mwv->font[j*16+i]);

			if(mwv->font[mwv->CurrentFont].polys)
			{
				HANDLE hx;

				FontOut(hdc, 16 * 16 + 50, 50, 1.0, &mwv->font[mwv->CurrentFont]);
				FontOut(hdc, 16 * 16 + 65, 50, 1.5, &mwv->font[mwv->CurrentFont]);
				FontOut(hdc, 16 * 16 + 85, 50, 2.0, &mwv->font[mwv->CurrentFont]);
				FontOut(hdc, 16 * 16 + 105, 50, 3.0, &mwv->font[mwv->CurrentFont]);

				if(curpoint)
				{
					RECT rc = {curpoint->x * 16 + 1, curpoint->y * 16 + 1, curpoint->x * 16 + 16, curpoint->y * 16 + 16};
					hx = CreateSolidBrush(RGB(255, 0, 0));
					FillRect(hdc, &rc, hx);
					DeleteObject(hx);
				}
				hx = SelectObject(hdc, CreatePen(PS_SOLID, 8, 0xFFFFFF));
				FontOut(hdc, 8, 8, 16.0, &mwv->font[mwv->CurrentFont]);
				DeleteObject(SelectObject(hdc, hx));
				hx = SelectObject(hdc, CreatePen(PS_SOLID, 8, RGB(255, 0, 0)));
				if(curpoly)
					FirePolylineTrans(hdc, curpoly->point, curpoly->points, 16.0, 8, 8);
				DeleteObject(SelectObject(hdc, hx));
			}
		}
		break;
	case WM_KEYDOWN:
		if(mwv->vk == VK_LEFT)
		{
			--mwv->CurrentFont == -1 ? mwv->CurrentFont = 255 : 0;
			curpoly = NULL;
			curpoint = NULL;
		}
		if(mwv->vk == VK_RIGHT)
		{
			++mwv->CurrentFont == 256 ? mwv->CurrentFont = 0 : 0;
			curpoly = NULL;
			curpoint = NULL;
		}
		if(mwv->vk == VK_LBUTTON)
		{
			if(mwv->mouse.x / 16 < 16 && mwv->mouse.y / 16 < 16)
			{
				if(curpoint)
				{
					curpoint->x = mwv->mouse.x / 16;
					curpoint->y = mwv->mouse.y / 16;
				}else
				{
					if(!curpoly)
					{
						struct FontChar* CurFont = &mwv->font[mwv->CurrentFont];
						CurFont->polys++;
						CurFont->poly = realloc(CurFont->poly, CurFont->polys * sizeof(struct FontCharPoly));
						curpoly = &CurFont->poly[mwv->font[mwv->CurrentFont].polys - 1];
						curpoly->points = 0;
						curpoly->point = NULL;
					}
					curpoly->points++;
					curpoly->point = realloc(curpoly->point, curpoly->points * sizeof(POINT));
					curpoly->point[curpoly->points - 1].x = mwv->mouse.x / 16;
					curpoly->point[curpoly->points - 1].y = mwv->mouse.y / 16;
				}
			}
			if(0 <= (mwv->mouse.x - 16*16-50) / 16 && 0 <= (mwv->mouse.y - 100) / 16 && (mwv->mouse.x - 16*16-50) / 16 < 16 && (mwv->mouse.y - 100) / 16 < 16)
			{
				mwv->CurrentFont = (mwv->mouse.y - 100) / 16 * 16 + (mwv->mouse.x - 16*16-50) / 16;
				curpoly = NULL;
				curpoint = NULL;
			}
		}
		if(mwv->vk == VK_MBUTTON)
		{
			if(mwv->mouse.x / 16 < 16 && mwv->mouse.y / 16 < 16)
			{
				struct FontChar* CurFont = &mwv->font[mwv->CurrentFont];
				int i, j, tries = -1;

FireReset:
				if(curpoint_lastchallangepoint.x == mwv->mouse.x / 16 && curpoint_lastchallangepoint.y == mwv->mouse.y / 16)
				{
					curpoint_challanges++;
				}else
				{
					curpoint_challanges = 0;
					curpoint_lastchallangepoint.x = mwv->mouse.x / 16;
					curpoint_lastchallangepoint.y = mwv->mouse.y / 16;
				}

				for(i = 0; i < CurFont->polys; i++)
				{
					for(j = 0; j < CurFont->poly[i].points; j++)
					{
						if(CurFont->poly[i].point[j].x == mwv->mouse.x / 16 && CurFont->poly[i].point[j].y == mwv->mouse.y / 16)
						{
							tries++;
							if(tries == curpoint_challanges)
							{
								curpoint = &CurFont->poly[i].point[j];
								curpoly = &CurFont->poly[i];
								goto FireQuit;
							}
						}
					}
				}
FireQuit:
				if(tries >= 0 && tries < curpoint_challanges)	// over
				{
					tries = -1;
					curpoint_challanges = -1;			// init
					goto FireReset;
				}
			}else
				curpoint = NULL;
		}
		if(mwv->vk == VK_RBUTTON)
		{
			if(curpoint)
				curpoint = NULL;
			else
				curpoly = NULL;
		}
		if(mwv->vk == VK_DELETE)
		{
			if(curpoly)
			{
				int i, j;

				if(curpoint)
				{
					i = curpoint - curpoly->point;	// pointer minus = already divided by sizeof(POINT)
					if(i < 0 || curpoly->points <= i)
						MessageBox(mwv->hWnd, "Internal Error: Not found curpoint", NULL, 0);
					else
					{
						for(j = i; j < curpoly->points - 1; j++)
							curpoly->point[j] = curpoly->point[j + 1];
						curpoint = NULL;
						curpoly->points--;
						if(curpoly->points == 0)
						{
							free(curpoly->point);
							curpoly->point = NULL;
							goto FireDelete;
						}
					}
				}else
				{
					struct FontChar* CurFont;
FireDelete:
					CurFont = &mwv->font[mwv->CurrentFont];
					i = curpoly - CurFont->poly;	// pointer minus = already divided by sizeof(struct FontCharPoly)
					if(i < 0 || CurFont->polys <= i)
						MessageBox(mwv->hWnd, "Internal Error: Not found curpoly", NULL, 0);
					else
					{
						if(curpoly->points)
							free(curpoly->point);
						for(j = i; j < CurFont->polys - 1; j++)
							CurFont->poly[j] = CurFont->poly[j + 1];
						curpoly = NULL;
						CurFont->polys--;
						if(CurFont->polys == 0)
						{
							free(CurFont->poly);
							CurFont->poly = NULL;
						}
					}
				}
			}
		}
		if(mwv->vk == 'S')
		{
			HANDLE hf;

			if(MessageBox(mwv->hWnd, "Overwrite?", NULL, MB_YESNO) == IDYES)
			{
				if(!(hf = CreateFileByExe("system.fnt", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL)))
					MessageBox(mwv->hWnd, "Internal Error: can't open system.fnt by Write", NULL, 0);
				else
				{
					int i, j, k;
					DWORD wrote;

					for(i = 0; i < 256; i++)
					{
						WriteFile(hf, &mwv->font[i].polys, sizeof(mwv->font[i].polys), &wrote, NULL);
						for(j = 0; j < mwv->font[i].polys; j++)
						{
							WriteFile(hf, &mwv->font[i].poly[j].points, sizeof(mwv->font[i].poly[j].points), &wrote, NULL);
							for(k = 0; k < mwv->font[i].poly[j].points; k++)
							{
								WriteFile(hf, &mwv->font[i].poly[j].point[k], sizeof(mwv->font[i].poly[j].point[k]), &wrote, NULL);
							}
						}
					}
					CloseHandle(hf);
				}
			}
		}
		if(mwv->vk == 'L')
		{
			LoadFont("system.fnt", mwv);
			curpoly = NULL;
			curpoint = NULL;
		}
		break;
	}
}
void PP_Logo(UINT msg, struct MainWndVal* mwv)
{
#define POW2(n) ((n)*(n))
#define STEP 2
#define TIME 20
#define PRESENTSSTEP 3
#define BLINKTIME 15
#define BLINKCOUNT 2
#define BLINKINTERVAL 1
#define PRESSTIME 25
#define PRESSINTERVAL 15
	const char murachue[] = "MURACHUE";
	const char presents[] = "PRESENTS";
	static unsigned int murachue_counter;
	static unsigned int presents_counter;
	static unsigned int blink_counter;
	static unsigned int press_counter;

	switch(msg)
	{
	case WM_CREATE:
		murachue_counter = TIME + (sizeof(murachue)-2)*STEP;
		presents_counter = 0;
		blink_counter = 0;
		press_counter = 0;
		break;
	case WM_PAINT:
		if(!blink_counter || (blink_counter < BLINKTIME) || (blink_counter - BLINKTIME) / BLINKINTERVAL % 2 == 1)
		{
			double pc = press_counter < PRESSTIME ? 1 : 1.0 - (double)(press_counter - PRESSTIME) / PRESSINTERVAL;
			{
				int mc;
				double zoom;
				int i;

				for(i = 0; i < sizeof(murachue) - 1; i++)
				{
					if(murachue_counter < TIME + (sizeof(murachue)-2-i)*STEP){
						mc = (sizeof(murachue)-2-i)*STEP <= murachue_counter && murachue_counter < TIME+(sizeof(murachue)-2-i)*STEP ? murachue_counter - (sizeof(murachue)-2-i)*STEP : 0;
						zoom = 2.0+POW2(POW2(mc / 10.0))/3.0;
						FontOutTransXY(mwv->hBackBuffer, 320+(int)((zoom*8+2)*(  -((int)(sizeof(murachue)-1)/2)  +i+(0.5-pc/2))), 240-16-(int)(zoom*8), zoom*pc, zoom, &mwv->font[murachue[i]]);
					}
				}
			}
			if(!murachue_counter)
			{
				int i;

				for(i = 0; i < (signed int)presents_counter / PRESENTSSTEP; i++)
					//if((presents_counter - (i-1)*PRESENTSSTEP)/2 != 5)
					FontOutTransXY(mwv->hBackBuffer, 320 - (17*(sizeof(presents)-1)/2) + (8+1+9)*i, 240, 0.7*pc, 0.7, &mwv->font[presents[i]]);
				if(presents_counter < PRESENTSSTEP*(sizeof(presents)-1))
					FontOut(mwv->hBackBuffer, 320 - (int)(17*((sizeof(presents)-1)/2 +(0.5-pc/2))) + (8+1+9)*i, 240, 0.7, &mwv->font['_']);
			}
		}
		break;
	case WM_TIMER:
		if(murachue_counter)
			murachue_counter--;
		if(!murachue_counter && presents_counter < PRESENTSSTEP*(sizeof(presents)-1))
			presents_counter++;
		if(presents_counter == PRESENTSSTEP*(sizeof(presents)-1) && blink_counter < BLINKTIME+(BLINKCOUNT*2+1)*BLINKINTERVAL)
			blink_counter++;
		if(blink_counter == BLINKTIME+(BLINKCOUNT*2+1)*BLINKINTERVAL && press_counter < PRESSTIME+PRESSINTERVAL)
			press_counter++;
		if(press_counter == PRESSTIME+PRESSINTERVAL)
			SetPhase(mwv, PHASE_TITLE);
		break;
	case WM_KEYDOWN:
		SetPhase(mwv, PHASE_TITLE);
		break;
	}
}
void PP_Title(UINT msg, struct MainWndVal* mwv)
{
	switch(msg)
	{
	case WM_PAINT:
		FontOutString(mwv->hBackBuffer, 20, 20, 2.50, mwv->font, "AKEOME2008");
		FontOutString(mwv->hBackBuffer, 60, 60, 0.75, mwv->font, "TENUKI DESUNE");

		FontOutString(mwv->hBackBuffer, 370, 460, 1.0, mwv->font, "PRESS ANY KEY OR MOUSE BUTTON");
		break;
	case WM_KEYDOWN:
		if(mwv->vk == VK_ESCAPE)
			SetPhase(mwv, PHASE_LOGO);
		else
			SetPhase(mwv, PHASE_GAME);
		break;
	}
}
#define ABS(x) ((x) < 0 ? -(x) : (x))
#define randf() (rand() / (double)RAND_MAX)
#define MSGLINES 45
void addmsg(char* msgs, int* msgc, int* msgx, char* msg)
{
	int l;

	for(l = MSGLINES - 2; 0 <= l; l--)
	{
		strcpy(msgs + (l + 1)*32, msgs + (l)*32);
		msgc[l + 1] = msgc[l];
		msgx[l + 1] = msgx[l];
	}
	sprintf(msgs, msg);
	msgc[0] = 30;
	msgx[0] = 0;
}

void PP_Game(UINT msg, struct MainWndVal* mwv)
{
	static int sleep;
	static int chain, chaincount;
	static int guardlife;
	static double level;
	static _int64 score;
	static char msgs[MSGLINES][32];
	static int msgc[MSGLINES];
	static int msgx[MSGLINES];
	static int quickcount, quickcombo, longcount;
	static BOOL gameoversounded;
#ifdef _DEBUG
	static BOOL _Dgoddefense = FALSE;
#endif

	switch(msg)
	{
	case WM_CREATE:
		sleep = 1;

		{
			int i;
			for(i = 0; i < 128; i++)
			{
				mwv->rock[i].use = FALSE;
				mwv->wind[i].use = FALSE;
			}
		}

		chain = 0;
		chaincount = 0;
		guardlife = 5;
#ifdef _DEBUG
		level = 0.3;
#else
		level = 0.01;
#endif
		score = 0;
		quickcount = 0;
		longcount = -1;
		gameoversounded = FALSE;

		{
			int i;

			for(i = 0; i < MSGLINES; i++)
			{
				msgs[i][0] = '\0';
				msgc[i] = 0;
				msgx[i] = 200;
			}
		}

		FMUSIC_PlaySong(mwv->bgm.Main);
		break;
	case WM_DESTROY:
		FMUSIC_StopSong(mwv->bgm.Main);
		break;
	case WM_PAINT:
		{
			HDC hdc = mwv->hBackBuffer;
			double i, j;

			// Draw Me!
			if(!sleep)
			{
				for(i = 12; i < 20; i++)
					Line(hdc, (int)(cos(i * PI/8 + mwv->me.at) * 10 + mwv->me.x), (int)(sin(i * PI/8 + mwv->me.at) * 10 + mwv->me.y), (int)(cos((i+1) * PI/8 + mwv->me.at) * 10 + mwv->me.x), (int)(sin((i+1) * PI/8 + mwv->me.at) * 10 + mwv->me.y));
				i=12;
				Line(hdc, (int)(cos(i * PI/8 + mwv->me.at)*10 + mwv->me.x), (int)(sin(i * PI/8 + mwv->me.at)*10 +  mwv->me.y), (int)(cos(i * PI/8 + mwv->me.at)*10 + mwv->me.x + 10*sin(PI/2*3 + mwv->me.at)), (int)(sin(i * PI/8 + mwv->me.at)*10 +  mwv->me.y + 10*-cos(PI/2*3 + mwv->me.at)));
				i=20;
				Line(hdc, (int)(cos(i * PI/8 + mwv->me.at)*10 + mwv->me.x), (int)(sin(i * PI/8 + mwv->me.at)*10 +  mwv->me.y), (int)(cos(i * PI/8 + mwv->me.at)*10 + mwv->me.x + 10*sin(PI/2*3 + mwv->me.at)), (int)(sin(i * PI/8 + mwv->me.at)*10 +  mwv->me.y + 10*-cos(PI/2*3 + mwv->me.at)));

				i=12, j=20;
				Line(hdc
					,(int)(cos(i * PI/8 + mwv->me.at)*10 + mwv->me.x + 10*sin(PI/2*3 + mwv->me.at)), (int)(sin(i * PI/8 + mwv->me.at)*10 +  mwv->me.y + 10*-cos(PI/2*3 + mwv->me.at))
					,(int)(cos(j * PI/8 + mwv->me.at)*10 + mwv->me.x + 10*sin(PI/2*3 + mwv->me.at)), (int)(sin(j * PI/8 + mwv->me.at)*10 +  mwv->me.y + 10*-cos(PI/2*3 + mwv->me.at)));
			}

			// Draw rocks!
			{
				int i, j;

				for(i = 0; i < 128; i++)
					if(mwv->rock[i].use)
					{
						POINT p[16];

						for(j = 0; j < 16; j++)
						{
							p[j].x = (int)(cos(j/16.0 * PI*2 + mwv->rock[i].rofs) * (mwv->rock[i].r + mwv->rock[i].bump[j]) + mwv->rock[i].x);
							p[j].y = (int)(sin(j/16.0 * PI*2 + mwv->rock[i].rofs) * (mwv->rock[i].r + mwv->rock[i].bump[j]) + mwv->rock[i].y);
						}
						Polygon(hdc, p, 16);
					}
			}

			// Draw winds!
			{
				int i;

				for(i = 0; i < 128; i++)
					if(mwv->wind[i].use)
						Ellipse(hdc, (int)(mwv->wind[i].x - mwv->wind[i].r * sqrt(2)), (int)(mwv->wind[i].y - mwv->wind[i].r * sqrt(2)), (int)(mwv->wind[i].x + mwv->wind[i].r * sqrt(2)), (int)(mwv->wind[i].y + mwv->wind[i].r * sqrt(2)));
			}

			if(guardlife)
				Ellipse(hdc, 320 - 125, 480 - 125, 320 + 125, 480 + 125);

			{
				int i;
				char ts[32];

				sprintf(ts, "SCORE: %I64d", score);
				FontOutString(hdc, 5, 5, 1.5, mwv->font, ts);

				if(chain >= 5)
				{
					sprintf(ts, "CHAIN: %u", chain);
					FontOutString(hdc, 5 - ((chaincount<30)?((30-chaincount)*3):0), 25, 1.0, mwv->font, ts);
				}

				for(i = 0; i < MSGLINES; i++)
					FontOutString(hdc, 5 - msgx[i]*2, 40 + i*14, 0.75, mwv->font, msgs[i]);

				if(!guardlife && sleep)
				{
					sprintf(ts, "GAME OVER");
					FontOutString(hdc, 260, 5, 2.0, mwv->font, ts);
					if(mwv->counter / 15 % 2 == 0)
					{
						sprintf(ts, "PRESS ESC");
						FontOutString(hdc, 270, 35, 1.0, mwv->font, ts);
					}
					sprintf(ts, "TENUKI DESU");
					FontOutString(hdc, 280, 55, 0.5, mwv->font, ts);
				}else
				{
					sprintf(ts, "LIFE: %u", guardlife);
					FontOutString(hdc, 280, 5, 2.0, mwv->font, ts);
				}

				sprintf(ts, "LEVEL: %2u/99", (int)(level * 100));
				FontOutString(hdc, 480, 5, 1.0, mwv->font, ts);

#ifdef _DEBUG
				sprintf(ts, "LONGCOUNT:%d", longcount);
				FontOutString(hdc, 5, 450, 0.75, mwv->font, ts);

				if(_Dgoddefense)
					FontOutString(hdc, 5, 465, 0.75, mwv->font, "GOD DEFENSE");
#endif
			}
		}
		break;
	case WM_TIMER:
		// THINK me
		if(GetAsyncKeyState(VK_LBUTTON) & 0x8000)
		{
			double mtome = atan2(mwv->mouse.y - mwv->me.y, mwv->mouse.x - mwv->me.x);
			
			double ax = cos(mtome);
			double ay = sin(mtome);
			double mx = cos(mwv->me.at);
			double my = sin(mwv->me.at);
			
			mwv->me.at = atan2((ay+my*10)/11, (ax+mx*10)/11);
			
			mwv->me.af += 0.3;
		}else
		{
			mwv->me.af -= 0.5;
			if(mwv->me.af < 0.5)
				mwv->me.af = 0.5;
		}
		mwv->me.x += cos(mwv->me.at) * mwv->me.af;
		mwv->me.y += sin(mwv->me.at) * mwv->me.af;
		if(!sleep && (mwv->me.x < 0 || mwv->me.y < 0 || 640 < mwv->me.x || 480 < mwv->me.y))
		{
			int i;

			sleep = 30;
			for(i = 0; i < 128 && mwv->wind[i].use; i++);
			if(i < 128)
			{
				mwv->wind[i].use = TRUE;
				mwv->wind[i].x = mwv->me.x;
				mwv->wind[i].y = mwv->me.y;
				mwv->wind[i].r = 0;
				mwv->wind[i].maxr = 100;
				mwv->wind[i].inval = TRUE;

				score -= 50;
				addmsg(&msgs[0][0], msgc, msgx, "PENALTY -50");

				wmPlayWave(mwv->wav.BoomWarn);
			}
		}

		// It's rebirth time?
		if(guardlife && sleep)
		{
			if(!--sleep)
			{
				mwv->me.x = 320.0;
				mwv->me.y = 460.0;
				mwv->me.af = 10;
				mwv->me.at = PI*3/2;

				wmPlayWave(mwv->wav.FireMain);
			}
		}

		// THINK rock
		{
			int i;

			for(i = 0; i < 128; i++)
				if(mwv->rock[i].use)
				{
					mwv->rock[i].x += mwv->rock[i].ax;
					mwv->rock[i].y += mwv->rock[i].ay;
					if(mwv->rock[i].ay < 5)
						mwv->rock[i].ay += mwv->rock[i].arofs;
					mwv->rock[i].rofs += mwv->rock[i].arofs;

					if(mwv->rock[i].x < -mwv->rock[i].r * 1.5 || 640 + mwv->rock[i].r * 1.5 < mwv->rock[i].x || 480 + mwv->rock[i].r * 1.5 < mwv->rock[i].y)
						mwv->rock[i].use = FALSE;

#ifdef _DEBUG
					if(!_Dgoddefense)
#endif
					if(guardlife && sqrt(POW2(mwv->rock[i].x - 320)+POW2(mwv->rock[i].y - 480)) < 125 + mwv->rock[i].r)
					{
						int k;

						mwv->rock[i].use = FALSE;
						for(k = 0; k < 128 && mwv->wind[k].use; k++);
						if(k < 128)
						{
							mwv->wind[k].use = TRUE;
							mwv->wind[k].x = mwv->rock[i].x;
							mwv->wind[k].y = mwv->rock[i].y;
							mwv->wind[k].r = 0;
							mwv->wind[k].maxr = 25;
							mwv->wind[k].plyrwind = FALSE;
							mwv->wind[i].inval = FALSE;
							if(guardlife)
							{
								guardlife--;
								if(guardlife)
								{
									wmPlayWave(mwv->wav.Guard);
									if(guardlife < 5)	// think future...
										wmPlayWave(mwv->wav.Life[guardlife]);
								}
								else
								{	// Bomb
									wmPlayWave(mwv->wav.RenBomb);
								}
							}
						}
					}

					if(!sleep && sqrt(POW2(mwv->me.x - mwv->rock[i].x)+POW2(mwv->me.y - mwv->rock[i].y)) < mwv->rock[i].r + 10)
					{
						int i;

						sleep = 30;
						for(i = 0; i < 128 && mwv->wind[i].use; i++);
						if(i < 128)
						{
							mwv->wind[i].use = TRUE;
							mwv->wind[i].x = mwv->me.x;
							mwv->wind[i].y = mwv->me.y;
							mwv->wind[i].r = 0;
							mwv->wind[i].maxr = 100;
							mwv->wind[i].inval = TRUE;

							score -= 50;
							addmsg(&msgs[0][0], msgc, msgx, "PENALTY -50");

							wmPlayWave(mwv->wav.BoomWarn);
						}
					}
				}
		}

		// THINK new_rock
		if(randf() < level)
		{
			int i;

			for(i = 0; i < 128; i++)
				if(!mwv->rock[i].use)
					break;

			if(i < 128)
			{
				int j;

				mwv->rock[i].use = TRUE;
				mwv->rock[i].x = randf() * 640;
				mwv->rock[i].r = 10 + randf()*15;
				mwv->rock[i].y = -mwv->rock[i].r;
				mwv->rock[i].ax = randf() * 2 - 1;
				mwv->rock[i].ay = 0;
				mwv->rock[i].rofs = 0;
				mwv->rock[i].arofs = 0;
				while(mwv->rock[i].arofs < 0.01)
					mwv->rock[i].arofs = randf() * PI/180*14 - PI/180*7;

				for(j = 0; j < 16; j++)
					mwv->rock[i].bump[j] = randf() * 4 - 2;
			}
		}

		// THINK wind
		{
			int i, j;
			BOOL boomed = FALSE;

			for(i = 0; i < 128; i++)
			{
				if(mwv->wind[i].use)
				{
					boomed = TRUE;
					mwv->wind[i].r += 10.0;

					// hittest
					if(!mwv->wind[i].inval)
						for(j = 0; j < 128; j++)
							if(mwv->rock[j].use)
							{
								double dx = mwv->rock[j].x - mwv->wind[i].x;
								double dy = mwv->rock[j].y - mwv->wind[i].y;
								if(sqrt(dx*dx + dy*dy) <= mwv->rock[j].r + mwv->wind[i].r)
								{
									int k;

									mwv->rock[j].use = FALSE;
									for(k = 0; k < 128 && mwv->wind[k].use; k++);
									if(k < 128)
									{
										mwv->wind[k].use = TRUE;
										mwv->wind[k].x = mwv->rock[j].x;
										mwv->wind[k].y = mwv->rock[j].y;
										mwv->wind[k].r = 0;
										if((mwv->wind[k].maxr = mwv->wind[i].maxr - 5) < 25)
											mwv->wind[k].maxr = 25;
										mwv->wind[k].plyrwind = FALSE;
										mwv->wind[k].inval = FALSE;

										(chain)++, chaincount = 50;
										if(chain == 5)
											wmPlayWave(mwv->wav.Chain);
										if(chain % 50 == 0)
										{
											char ts[32];

											wmPlayWave(mwv->wav.ChainBonus);
											score += chain/50*1000;
											sprintf(ts, "CHAIN BONUS +%u", chain/50*1000);
											addmsg(&msgs[0][0], msgc, msgx, ts);
										}
										if(quickcount)
											quickcombo++;

										wmPlayWave(mwv->wav.RenBomb);

										score += POW2(chain);

										if(mwv->wind[i].plyrwind && mwv->wind[i].r <= 15)
										{
											wmPlayWave(mwv->wav.NExp);
											score += 50000;
											addmsg(&msgs[0][0], msgc, msgx, "NEAR EXPLOSION +50000");
										}
										if(mwv->wind[i].r >= mwv->wind[i].maxr)
										{
											wmPlayWave(mwv->wav.FExp);
											score += 10;
											addmsg(&msgs[0][0], msgc, msgx, "FAR EXPLOSION +10");
										}
										if(guardlife && sqrt(POW2(mwv->rock[j].x - 320)+POW2(mwv->rock[j].y - 480)) < 125+25 + mwv->rock[j].r)
										{
											wmPlayWave(mwv->wav.GuardedBase);
											score += 200;
											addmsg(&msgs[0][0], msgc, msgx, "GUARD BASE +200");
										}
									}
								}
							}

					if(mwv->wind[i].r >= mwv->wind[i].maxr)
						mwv->wind[i].use = FALSE;
				}
			}

			if(!boomed)
				longcount = -1;
			else if(longcount > 0)
			{
				if(!--longcount)
				{
					wmPlayWave(mwv->wav.LongSweep);
					score += 10000;
					addmsg(&msgs[0][0], msgc, msgx, "LONG SWEEP +10000");
				}
			}
		}

		// THINK chain
		if(chaincount)
		{
			chaincount--;
			if(!chaincount)
				chain = 0;
		}

		// THINK msgs
		{
			int i;

			for(i = 0; i < MSGLINES; i++)
				if(msgs[i][0] != '\0')
					if(msgx[i] < (signed)strlen(msgs[i])*7+5)
					{
						if(msgc[i])
							msgc[i]--;
						else
							msgx[i]++;
					}else
						msgs[i][0] = '\0';
		}

		// THINK counts
		if(quickcount)
		{
			if(!--quickcount)
			{
				if(quickcombo >= 30)
				{
					wmPlayWave(mwv->wav.QuickSweep);
					score += 5000;
					addmsg(&msgs[0][0], msgc, msgx, "QUICK SWEEP +5000");
				}
			}
		}

		// THINK gameovered
		if(!gameoversounded && !guardlife && sleep)
		{
			gameoversounded = TRUE;
			wmPlayWave(mwv->wav.GameOver);
		}

		if(level <= 0.99)
			level += 0.0001;

		break;
	case WM_KEYDOWN:
		if(mwv->vk == VK_ESCAPE)
			SetPhase(mwv, PHASE_TITLE);
		if(mwv->vk == VK_LBUTTON)
			if(!sleep)
				wmPlayWave(mwv->wav.Accel);
		if(mwv->vk == VK_RBUTTON)
			if(!sleep)
			{
				if(sqrt(POW2(mwv->me.x - 320)+POW2(mwv->me.y - 480)) < 125 + 10)
				{
					wmPlayWave(mwv->wav.Invalid);
				}else
				{
					int i;
					sleep = 30 - (int)(level * 20);
					for(i = 0; i < 128 && mwv->wind[i].use; i++);
					if(i < 128)
					{
						mwv->wind[i].use = TRUE;
						mwv->wind[i].x = mwv->me.x;
						mwv->wind[i].y = mwv->me.y;
						mwv->wind[i].r = 0;
						mwv->wind[i].maxr = 100;
						mwv->wind[i].plyrwind = TRUE;
						mwv->wind[i].inval = FALSE;
					}
					quickcount = 15;
					quickcombo = 0;
					if(longcount == -1)
						longcount = 90;
					wmPlayWave(mwv->wav.Bomb);
				}
			}
#ifdef _DEBUG
		if(mwv->vk == 'Z')
		{
			KillTimer(mwv->hWnd, 0);
			SetTimer(mwv->hWnd, 0, 1000/10, NULL);
		}
		if(mwv->vk == 'X')
		{
			KillTimer(mwv->hWnd, 0);
			SetTimer(mwv->hWnd, 0, 1000/30, NULL);
		}
		if(mwv->vk == 'C')
		{
			_Dgoddefense = !_Dgoddefense;
		}
		if(mwv->vk == 'V')
		{
			level -= 0.1;
			if(level < 0.01)
				level = 0.01;
		}
		if(mwv->vk == 'B')
		{
			level += 0.1;
			if(0.99 < level)
				level = 0.99;
		}
#endif
		break;
	}
}

void (*PhaseProcess[])(UINT msg, struct MainWndVal* mwv) = {
	PP_Null	// dummy
	,	// VC++ indent
	PP_FontEdit,
	PP_Logo,
	PP_Title,
	PP_Game,
};

LRESULT CALLBACK MainWndProc(
    HWND hWnd,	// handle of window
    UINT msg,	// message identifier
    WPARAM wParam,	// first message parameter
    LPARAM lParam 	// second message parameter
	)
{
	switch(msg)
	{
	case WM_CREATE:
		{
			struct MainWndVal* mwv;

			if(!(mwv = malloc(sizeof(*mwv))))
				return -1;
			SetWindowLong(hWnd, GWL_USERDATA, (LONG)mwv);

			mwv->hWnd = hWnd;

			// Init backbuffer
			{
				HDC hdc;

				hdc = GetDC(hWnd);
				// ȉWM_SIZEp_~[
				mwv->hBackBuffer = CreateCompatibleDC(hdc);
				mwv->hFirstBackBufferBitmap = SelectObject(mwv->hBackBuffer, CreateCompatibleBitmap(hdc, 0, 0));
				ReleaseDC(hWnd, hdc);
			}

			// Init system
			mwv->mouse.x = mwv->mouse.y = -1;
			{
				int i;

				for(i = 0; i < 256; i++)
				{
					mwv->font[i].polys = 0;
					mwv->font[i].poly = NULL;
				}
			}
			mwv->phase = PHASE_NULL;
			if(GetAsyncKeyState(VK_CONTROL) & 0x8000)
				SetPhase(mwv, PHASE_FONTEDIT);
			else
				SetPhase(mwv, PHASE_LOGO);
			LoadFont(NULL, mwv);

			mwv->wav.FireMain = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WFIREMAIN), "WAVE"))));
			mwv->wav.Accel = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WACCEL), "WAVE"))));
			mwv->wav.Bomb = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WBOMB), "WAVE"))));
			mwv->wav.RenBomb = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WRENBOMB), "WAVE"))));
			mwv->wav.BoomWarn = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WBOOMWARN), "WAVE"))));
			mwv->wav.Invalid = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WINVALID), "WAVE"))));
			mwv->wav.Guard = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WGUARD), "WAVE"))));
			mwv->wav.NExp = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WNE), "WAVE"))));
			mwv->wav.FExp = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WFE), "WAVE"))));
			mwv->wav.Chain = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WCHAIN), "WAVE"))));
			mwv->wav.Life[0] = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_W0), "WAVE"))));
			mwv->wav.Life[1] = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_W1), "WAVE"))));
			mwv->wav.Life[2] = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_W2), "WAVE"))));
			mwv->wav.Life[3] = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_W3), "WAVE"))));
			mwv->wav.Life[4] = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_W4), "WAVE"))));
			mwv->wav.ChainBonus = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WCHAINBONUS), "WAVE"))));
			mwv->wav.GameOver = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WGAMEOVER), "WAVE"))));
			mwv->wav.GuardedBase = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WGUARDEDBASE), "WAVE"))));
			mwv->wav.QuickSweep = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WQUICKSWEEP), "WAVE"))));
			mwv->wav.LongSweep = wmMemoryWave(LockResource(LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(IDR_WLONGSWEEP), "WAVE"))));

			//mwv->bgm.Main = FMUSIC_LoadSong(MAKEINTRESOURCE(IDR_BMAIN), NULL);
			{
				HRSRC rsc = FindResource(NULL, MAKEINTRESOURCE(IDR_BMAIN), "MOD");
				mwv->bgm.Main = FMUSIC_LoadSongEx( LockResource(LoadResource(NULL, rsc)), 0, SizeofResource(NULL, rsc), FSOUND_LOADMEMORY, NULL, 0);
				//mwv->bgm.Main = FMUSIC_LoadSongEx( "z_bpuls.it", 0, 0, 0, NULL, 0);
			}

			// Hide cursor
			ShowCursor(FALSE);

			// Pump
			SetTimer(hWnd, 0, 1000/30, NULL);
		}
		return 0;
	case WM_SIZE:
		if(
			wParam == SIZE_MINIMIZED ||
			wParam == SIZE_MAXHIDE// ||
			//wParam == SIZE_RESTORED
			)
			return 0;

		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);
			HDC hdc;

			// Delete old backbuffer
			DeleteObject(SelectObject(mwv->hBackBuffer, mwv->hFirstBackBufferBitmap));
			DeleteDC(mwv->hBackBuffer);

			// Create new backbuffer
			hdc = GetDC(hWnd);
			mwv->hBackBuffer = CreateCompatibleDC(hdc);
			mwv->hFirstBackBufferBitmap = SelectObject(mwv->hBackBuffer, CreateCompatibleBitmap(hdc, LOWORD(lParam), HIWORD(lParam)));
			ReleaseDC(hWnd, hdc);

			mwv->wsize.cx = LOWORD(lParam);
			mwv->wsize.cy = HIWORD(lParam);

			ClipCursorToWindowClient(hWnd, TRUE);
		}
		return 0;
	case WM_ACTIVATE:
		{
			//struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);

			if(LOWORD(wParam) == WA_INACTIVE)
				ClipCursorToWindowClient(hWnd, FALSE);
			if(LOWORD(wParam) == WA_CLICKACTIVE || LOWORD(wParam) == WA_ACTIVE)
			{
				ClipCursorToWindowClient(hWnd, TRUE);
				if(LOWORD(wParam) == WA_CLICKACTIVE)
				{
					MSG msg;
					PeekMessage(&msg, hWnd, WM_LBUTTONDOWN, WM_LBUTTONDOWN, PM_REMOVE);
					PeekMessage(&msg, hWnd, WM_MBUTTONDOWN, WM_MBUTTONDOWN, PM_REMOVE);
					PeekMessage(&msg, hWnd, WM_RBUTTONDOWN, WM_RBUTTONDOWN, PM_REMOVE);
				}
			}
		}
		return 0;
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC xhdc;

			xhdc = BeginPaint(hWnd, &ps);
			{
				struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);
				HDC hdc = mwv->hBackBuffer;	// alias mwv->hBackBuffer as hdc

				SelectObject(hdc, GetStockObject(NULL_PEN));
				SelectObject(hdc, GetStockObject(BLACK_BRUSH));
				Rectangle(hdc, 0, 0, mwv->wsize.cx + 1, mwv->wsize.cy + 1);

				SelectObject(hdc, GetStockObject(WHITE_PEN));
				SelectObject(hdc, GetStockObject(NULL_BRUSH));

				PhaseProcess[mwv->phase](msg, mwv);

				{
					POINT m = mwv->mouse;
					POINT p[] = {
						{m.x + 0, m.y + 0},
						{m.x + 10, m.y + 10},
						{m.x + 5, m.y + 10},
						{m.x + 0, m.y + 15},
					};
					Polygon(hdc, p, sizeof(p) / sizeof(POINT));
				}

				BitBlt(xhdc, 0, 0, mwv->wsize.cx, mwv->wsize.cy, hdc, 0, 0, SRCCOPY);
			}
			EndPaint(hWnd, &ps);
		}
		return 0;
	case WM_TIMER:
		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);
			POINT m;

			mwv->counter++;

			GetCursorPos(&m);
			ScreenToClient(hWnd, &m);
			if(m.x < 0 || m.y < 0 || mwv->wsize.cx <= m.x || mwv->wsize.cy <= m.y)
			{
				mwv->outmouse = TRUE;
				//mwv->mouse.x = mwv->mouse.y = -1;
			}else
			{
				mwv->outmouse = FALSE;
				mwv->mouse = m;
			}

			PhaseProcess[mwv->phase](msg, mwv);

			InvalidateRect(hWnd, NULL, FALSE);
		}
		return 0;
	case WM_KEYDOWN:
	case WM_KEYUP:
		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);

			mwv->vk = (int)wParam;
			PhaseProcess[mwv->phase](msg, mwv);
		}
		break;
	case WM_LBUTTONDOWN:
	case WM_MBUTTONDOWN:
	case WM_RBUTTONDOWN:
		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);

			if(msg == WM_LBUTTONDOWN)	mwv->vk = VK_LBUTTON;
			if(msg == WM_MBUTTONDOWN)	mwv->vk = VK_MBUTTON;
			if(msg == WM_RBUTTONDOWN)	mwv->vk = VK_RBUTTON;
			PhaseProcess[mwv->phase](WM_KEYDOWN, mwv);
		}
		break;
	case WM_LBUTTONUP:
	case WM_MBUTTONUP:
	case WM_RBUTTONUP:
		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);

			if(msg == WM_LBUTTONUP)	mwv->vk = VK_LBUTTON;
			if(msg == WM_MBUTTONUP)	mwv->vk = VK_MBUTTON;
			if(msg == WM_RBUTTONUP)	mwv->vk = VK_RBUTTON;
			PhaseProcess[mwv->phase](WM_KEYUP, mwv);
		}
		break;
	case WM_CLOSE:
		PostQuitMessage(0);
		break;
	case WM_DESTROY:
		{
			struct MainWndVal* mwv = (void*)GetWindowLong(hWnd, GWL_USERDATA);

			KillTimer(hWnd, 0);

			SetPhase(mwv, PHASE_NULL);

			DeleteObject(SelectObject(mwv->hBackBuffer, mwv->hFirstBackBufferBitmap));
			DeleteDC(mwv->hBackBuffer);

			{
				int i;

				for(i = 0; i < 256; i++)
				{
					if(mwv->font[i].polys && mwv->font[i].poly)
					{
						int j;

						for(j = 0; j < mwv->font[i].polys; j++)
						{
							if(mwv->font[i].poly[j].points && mwv->font[i].poly[j].point)
								free(mwv->font[i].poly[j].point);
						}
						free(mwv->font[i].poly);
					}
				}
			}

			wmKillAllWave();
			wmUnloadWave(mwv->wav.FireMain);
			wmUnloadWave(mwv->wav.Accel);
			wmUnloadWave(mwv->wav.Bomb);
			wmUnloadWave(mwv->wav.RenBomb);
			wmUnloadWave(mwv->wav.BoomWarn);
			wmUnloadWave(mwv->wav.Invalid);
			wmUnloadWave(mwv->wav.Guard);
			wmUnloadWave(mwv->wav.NExp);
			wmUnloadWave(mwv->wav.FExp);
			wmUnloadWave(mwv->wav.Chain);

			wmUnloadWave(mwv->wav.Life[0]);
			wmUnloadWave(mwv->wav.Life[1]);
			wmUnloadWave(mwv->wav.Life[2]);
			wmUnloadWave(mwv->wav.Life[3]);
			wmUnloadWave(mwv->wav.Life[4]);
			wmUnloadWave(mwv->wav.ChainBonus);
			wmUnloadWave(mwv->wav.GameOver);
			wmUnloadWave(mwv->wav.GuardedBase);
			wmUnloadWave(mwv->wav.QuickSweep);
			wmUnloadWave(mwv->wav.LongSweep);

			FMUSIC_FreeSong(mwv->bgm.Main);

			free(mwv);
		}
		break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

// CopyPaste Oisiidesu
int WINAPI WinMain(
	HINSTANCE hInst,	// handle to current instance
    HINSTANCE hPrevInst,	// handle to previous instance
    LPSTR lpCmdLine,	// pointer to command line
    int nCmdShow 	// show state of window
	)
{
	WNDCLASSEX wc = {0};
	HWND hMainWnd;
	MSG msg;

	srand(GetTickCount());

	if(wmInitSystem())
	{
		MessageBox(NULL, "WaveOut initialize failed!\nExit.", "FATAL Error", MB_ICONERROR | MB_OK);
		return 1;
	}

	//FSOUND_File_SetCallbacks(memopen, memclose, memread, memseek, memtell);
	FSOUND_Init(44100,8,0);

	wc.cbSize = sizeof(wc);
	wc.hInstance = hInst;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);
	wc.lpfnWndProc = MainWndProc;
	wc.lpszClassName = "Ake2008MainWndClass";
	if(!RegisterClassEx(&wc))
		return 1;
	if(!(hMainWnd = CreateWindowEx(0, wc.lpszClassName, "ake2008", WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX,
		0, 0, 0, 0, NULL, NULL, hInst, NULL)))
		return 1;

	{
		RECT rc, src;

		rc.left = 0;
		rc.top = 0;
		rc.right = 640;
		rc.bottom = 480;
		AdjustWindowRectEx(&rc, GetWindowLong(hMainWnd, GWL_STYLE), GetMenu(hMainWnd) ? TRUE : FALSE, GetWindowLong(hMainWnd, GWL_EXSTYLE));
		SystemParametersInfo(SPI_GETWORKAREA, 0, &src, 0);
		SetWindowPos(hMainWnd, NULL,
			src.left + (src.right - src.left)/2 - (rc.right - rc.left)/2, src.top + (src.bottom - src.top)/2 - (rc.bottom - rc.top)/2,
			rc.right - rc.left, rc.bottom - rc.top, SWP_NOZORDER);
		ShowWindow(hMainWnd, nCmdShow);
	}

	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	wmUninitSystem();
	FSOUND_Close();

	return msg.wParam;
}
